@@ -30,6 +30,7 @@
 
 /*#define GWEN_TLS_DEBUG*/
 
+/* #define GWEN_TLS_USE_OLD_CODE */
 
 #include "syncio_tls_p.h"
 #include "i18n_l.h"
@@ -327,6 +328,8 @@ int GWEN_SyncIo_Tls_Prepare(GWEN_SYNCIO *sio) {
   GWEN_SYNCIO_TLS *xio;
   int rv;
   uint32_t lflags;
+  const char *custom_ciphers;
+  const char *errPos=NULL;
 
   assert(sio);
   xio=GWEN_INHERIT_GETDATA(GWEN_SYNCIO, GWEN_SYNCIO_TLS, sio);
@@ -349,111 +352,27 @@ int GWEN_SyncIo_Tls_Prepare(GWEN_SYNCIO *sio) {
     return GWEN_ERROR_GENERIC;
   }
 
-  /* set default priority */
-#if 1 /* old code */
-  GWEN_Gui_ProgressLog(0, GWEN_LoggerLevel_Error, I18N("Using old SSL preparation code."));
-  rv=gnutls_set_default_priority(xio->session);
-  if (rv) {
-    DBG_ERROR(GWEN_LOGDOMAIN, "gnutls_set_default_priority: %d (%s)", rv, gnutls_strerror(rv));
-    gnutls_deinit(xio->session);
-    return GWEN_ERROR_GENERIC;
-  }
-
-  /* possibly force protocol priority */
-  if (lflags & GWEN_SYNCIO_TLS_FLAGS_FORCE_SSL_V3) {
-    char *force_sslv3_str = "NORMAL:-VERS-TLS-ALL:+VERS-SSL3.0";
-
-    DBG_INFO(GWEN_LOGDOMAIN, "Forcing SSL v3");
-    rv=gnutls_priority_set_direct(xio->session, force_sslv3_str, NULL);
-    if (rv) {
-      DBG_ERROR(GWEN_LOGDOMAIN, "gnutls_priority_set_direct: %d (%s)", rv, gnutls_strerror(rv));
-      gnutls_deinit(xio->session);
-      return GWEN_ERROR_GENERIC;
-    }
-  }
-#else /* new code */
-  {
-    /* TODO: The following does not work with all servers, disabled for now  */
-    const char *errPos=NULL;
-    const char *ciphers_default;
-    GWEN_BUFFER *ciphers = NULL;
-
-    GWEN_Gui_ProgressLog(0, GWEN_LoggerLevel_Error, I18N("Using new SSL preparation code."));
-    ciphers=GWEN_Buffer_new(0, 256, 0, 1);
-    assert(ciphers!=NULL);
-
-    ciphers_default=getenv("GWEN_TLS_CIPHER_PRIORITIES");
-    if (ciphers_default && *ciphers_default) { /* use cipher list from env var */
-      rv=GWEN_Buffer_AppendString(ciphers, ciphers_default);
-      if (rv!=0) {
-        DBG_ERROR(GWEN_LOGDOMAIN, "failed to append default ciphers to cipher list: %d", rv);
-        GWEN_Buffer_free(ciphers);
-        gnutls_deinit(xio->session);
-        return GWEN_ERROR_GENERIC;
-      }
-    }
-    else { /* use hardcoded default ciphers */
-
-      /* this might get removed later, because it is no longer needed */
-      if (lflags & GWEN_SYNCIO_TLS_FLAGS_FORCE_SSL_V3) {
-        rv=GWEN_Buffer_AppendString(ciphers, "+VERS-SSL3.0:");
-        if (rv!=0) {
-          DBG_ERROR(GWEN_LOGDOMAIN, "failed to append SSLv3 prefix to cipher list: %d", rv);
-          GWEN_Buffer_free(ciphers);
-          gnutls_deinit(xio->session);
-          return GWEN_ERROR_GENERIC;
-        }
-      }
-
-      if (lflags & GWEN_SYNCIO_TLS_FLAGS_FORCE_UNSAFE_CIPHERS) {
-        GWEN_Gui_ProgressLog(0, GWEN_LoggerLevel_Error, I18N("TLS: Warning - Forcing unsafe SSL-Ciphers per user request."));
-        /* add hardcoded 128-Bit-ciphers ciphers */
-        rv=GWEN_Buffer_AppendString(ciphers, GWEN_TLS_CIPHER_PRIORITIES_UNSAFE);
-        if (rv!=0) {
-          DBG_ERROR(GWEN_LOGDOMAIN, "failed to append default ciphers to cipher list: %d", rv);
-          GWEN_Buffer_free(ciphers);
-          gnutls_deinit(xio->session);
-          return GWEN_ERROR_GENERIC;
-        }
-      }
-      else {
-        /* add hardcoded default ciphers */
-        rv=GWEN_Buffer_AppendString(ciphers, GWEN_TLS_CIPHER_PRIORITIES_DEFAULT);
-        if (rv!=0) {
-          DBG_ERROR(GWEN_LOGDOMAIN, "failed to append default ciphers to cipher list: %d", rv);
-          GWEN_Buffer_free(ciphers);
-          gnutls_deinit(xio->session);
-          return GWEN_ERROR_GENERIC;
-        }
-
-        /* add list of disabled ciphers */
-        if (lflags & GWEN_SYNCIO_TLS_FLAGS_ONLY_SAFE_CIPHERS) {
-          DBG_INFO(GWEN_LOGDOMAIN, "Removing unsafe ciphers");
-          GWEN_Gui_ProgressLog(0, GWEN_LoggerLevel_Info, I18N("TLS: Disabling unsafe ciphers per user request."));
-          rv=GWEN_Buffer_AppendString(ciphers, ":"GWEN_TLS_CIPHER_PRIORITIES_DISABLE_UNSAFE);
-          if (rv!=0) {
-            DBG_ERROR(GWEN_LOGDOMAIN, "failed to append unsafe ciphers to cipher list: %d", rv);
-            GWEN_Buffer_free(ciphers);
-            gnutls_deinit(xio->session);
-            return GWEN_ERROR_GENERIC;
-          }
-        }
-      }
-    }
-
-    DBG_INFO(GWEN_LOGDOMAIN, "Setting cipher priority to [%s]", GWEN_Buffer_GetStart(ciphers));
-    GWEN_Gui_ProgressLog2(0, GWEN_LoggerLevel_Info, I18N("TLS: SSL-Cipher priority list: %s"), GWEN_Buffer_GetStart(ciphers));
-    rv=gnutls_priority_set_direct(xio->session, GWEN_Buffer_GetStart(ciphers), &errPos);
+  /* set cipher priorities */
+  custom_ciphers=getenv("GWEN_TLS_CIPHER_PRIORITIES");
+  /* TODO: make custom ciphers configurable as priority string? */
+  if (custom_ciphers && *custom_ciphers) { /* use cipher list from env var */
+    GWEN_Gui_ProgressLog2(0, GWEN_LoggerLevel_Info, I18N("TLS: SSL cipher priority list: %s"), custom_ciphers);
+    rv=gnutls_priority_set_direct(xio->session, custom_ciphers, &errPos);
     if (rv!=GNUTLS_E_SUCCESS) {
-      DBG_ERROR(GWEN_LOGDOMAIN, "gnutls_priority_set_direct using '%s' failed: %d (%s) [%s]",
-                GWEN_Buffer_GetStart(ciphers), rv, gnutls_strerror(rv), errPos?errPos:"");
-      GWEN_Buffer_free(ciphers);
+      DBG_ERROR(GWEN_LOGDOMAIN, "gnutls_priority_set_direct using '%s' failed: %s (%d) [%s]",
+                custom_ciphers, gnutls_strerror(rv), rv, errPos?errPos:"");
+      gnutls_deinit(xio->session);
+      return GWEN_ERROR_GENERIC;
+    }
+  } else { /* use default ciphers from GnuTLS */
+    GWEN_Gui_ProgressLog(0, GWEN_LoggerLevel_Notice, I18N("Using GnuTLS default ciphers."));
+    rv=gnutls_set_default_priority(xio->session);
+    if (rv!=GNUTLS_E_SUCCESS) {
+      DBG_ERROR(GWEN_LOGDOMAIN, "gnutls_set_default_priority failed: %s (%d)", gnutls_strerror(rv), rv);
       gnutls_deinit(xio->session);
       return GWEN_ERROR_GENERIC;
     }
-    GWEN_Buffer_free(ciphers);
   }
-#endif
 
   /* protect against too-many-known-ca problem */
   gnutls_handshake_set_max_packet_length(xio->session, 64*1024);
@@ -1242,7 +1161,7 @@ int GWENHYWFAR_CB GWEN_SyncIo_Tls_Connect(GWEN_SYNCIO *sio) {
   do {
     rv=gnutls_handshake(xio->session);
   }
-  while (rv==GNUTLS_E_AGAIN && rv==GNUTLS_E_INTERRUPTED);
+  while (rv==GNUTLS_E_AGAIN || rv==GNUTLS_E_INTERRUPTED);
 
   if (rv) {
     DBG_ERROR(GWEN_LOGDOMAIN, "gnutls_handshake: %d (%s) [%s]",
@@ -1333,7 +1252,7 @@ int GWENHYWFAR_CB GWEN_SyncIo_Tls_Disconnect(GWEN_SYNCIO *sio) {
   do {
     rv=gnutls_bye(xio->session, GNUTLS_SHUT_RDWR);
   }
-  while (rv==GNUTLS_E_AGAIN && rv==GNUTLS_E_INTERRUPTED);
+  while (rv==GNUTLS_E_AGAIN || rv==GNUTLS_E_INTERRUPTED);
 
   if (rv) {
     DBG_ERROR(GWEN_LOGDOMAIN, "gnutls_bye: %d (%s)", rv, gnutls_strerror(rv));
@@ -1380,7 +1299,7 @@ int GWENHYWFAR_CB GWEN_SyncIo_Tls_Read(GWEN_SYNCIO *sio,
   do {
     rv=gnutls_record_recv(xio->session, buffer, size);
   }
-  while (rv==GNUTLS_E_AGAIN && rv==GNUTLS_E_INTERRUPTED);
+  while (rv==GNUTLS_E_AGAIN || rv==GNUTLS_E_INTERRUPTED);
 
   if (rv<0) {
     DBG_ERROR(GWEN_LOGDOMAIN, "gnutls_record_recv: %d (%s)", rv, gnutls_strerror(rv));
@@ -1394,6 +1313,11 @@ int GWENHYWFAR_CB GWEN_SyncIo_Tls_Read(GWEN_SYNCIO *sio,
     GWEN_SyncIo_SetStatus(sio, GWEN_SyncIo_Status_Disconnected);
     GWEN_SyncIo_Tls_UndoPrepare(sio);
     GWEN_SyncIo_Disconnect(baseIo);
+#ifdef GNUTLS_E_PREMATURE_TERMINATION
+    if (rv==GNUTLS_E_PREMATURE_TERMINATION) {
+      return GWEN_ERROR_SSL_PREMATURE_CLOSE;
+    }
+#endif
     return GWEN_ERROR_SSL;
   }
 
@@ -1436,7 +1360,7 @@ int GWENHYWFAR_CB GWEN_SyncIo_Tls_Write(GWEN_SYNCIO *sio,
   do {
     rv=gnutls_record_send(xio->session, buffer, size);
   }
-  while (rv==GNUTLS_E_AGAIN && rv==GNUTLS_E_INTERRUPTED);
+  while (rv==GNUTLS_E_AGAIN || rv==GNUTLS_E_INTERRUPTED);
 
   if (rv<0) {
     DBG_ERROR(GWEN_LOGDOMAIN, "gnutls_record_send: %d (%s)", rv, gnutls_strerror(rv));
